package com.biganto.visual.roompark.domain.use_case

//import io.reactivex.Observable
import com.biganto.visual.roompark.data.repository.db.requrey.model.SubscriptionEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.DeviceUtilsContract
import com.biganto.visual.roompark.domain.contract.SubscriptionContract
import io.reactivex.rxkotlin.Observables
import javax.inject.Inject
//import io.reactivex.rxkotlin

/**
 * Created by Vladislav Bogdashkin on 01.11.2019.
 */

class SubscriptionUseCase @Inject constructor(
    private val subscription: SubscriptionContract,
    private val utils: DeviceUtilsContract,
    private val auth: AuthContract
){
    fun subscribeTopic(subId:Int,topic:SubscriptionTopic) =
        Observables.zip(auth.currentUser(),utils.getDeviceId()){ user, token ->
            subscription.subscribeTopic(
                user
                ,subId
                ,token
                ,topic = topic.topicName,
                topic_id = topic.topicId
            )
        }

    fun unSubscribeTopic(subId:Int,topic:SubscriptionTopic) =
        Observables.zip(auth.currentUser(),utils.getDeviceId()){ user, token ->
            subscription.unSubscribeTopic(user
                ,subId
                ,token
                ,topic = topic.topicName,
                topic_id = topic.topicId)
        }

    fun getSubscriptions(topic:SubscriptionTopic) = auth.currentUser()
        .map {
            var sub = it.subscriptions
                ?.firstOrNull{it.topic == topic.topicName && it.number == topic.topicId}
            if (sub == null) {
                sub = SubscriptionEntity()
                sub.setOwner(it)
                sub.setTopic(topic.topicName)
                sub.setNumber(topic.topicId)
                sub.setState(false)
            }
            subscription.saveSubscribeState(sub as SubscriptionEntity).blockingFirst()
        }

//    fun getSubscription()

}


sealed class SubscriptionTopic(val topicName:String,val topicId:String? = null) {
    class Deals(topicName: String =TOPIC_API_NAME_DEALS,dealId:String?) : SubscriptionTopic(topicName,dealId)
    class Progress_1(topicName: String =TOPIC_API_NAME_PROGRESS_1) : SubscriptionTopic(topicName)
    class Progress_2(topicName: String =TOPIC_API_NAME_PROGRESS_2) : SubscriptionTopic(topicName)
    class Progress_3(topicName: String =TOPIC_API_NAME_PROGRESS_3) : SubscriptionTopic(topicName)
    class Progress_kindergarden(topicName: String =TOPIC_API_NAME_PROGRESS_KINDERGARDEN) : SubscriptionTopic(topicName)
    class Progress_school(topicName: String =TOPIC_API_NAME_PROGRESS_SCHOOL) : SubscriptionTopic(topicName)
    class Progress_landscaping(topicName: String =TOPIC_API_NAME_PROGRESS_LANDSCAPING) : SubscriptionTopic(topicName)
    class News(topicName: String =TOPIC_API_NAME_NEWS) : SubscriptionTopic(topicName)
    class Blog(topicName: String =TOPIC_API_NAME_BLOG) : SubscriptionTopic(topicName)
    class Construction(topicName: String = TOPIC_API_NAME_CONSCTRUCTION) : SubscriptionTopic(topicName)

}

private const val TOPIC_API_NAME_DEALS = "deals"
private const val TOPIC_API_NAME_PROGRESS_1 = "progress-1"
private const val TOPIC_API_NAME_PROGRESS_2 = "progress-2"
private const val TOPIC_API_NAME_PROGRESS_3 = "progress-3"
private const val TOPIC_API_NAME_PROGRESS_KINDERGARDEN = "progress-kindergarden"
private const val TOPIC_API_NAME_PROGRESS_SCHOOL = "progress-school"
private const val TOPIC_API_NAME_PROGRESS_LANDSCAPING = "progress-landscaping"
private const val TOPIC_API_NAME_NEWS = "news"
private const val TOPIC_API_NAME_BLOG = "blog"
private const val TOPIC_API_NAME_CONSCTRUCTION = "construction-blog"

private fun fromEntity(sub:SubscriptionEntity):SubscriptionTopic{
    when(sub.topic){
        TOPIC_API_NAME_DEALS -> SubscriptionTopic.Deals(dealId = sub.number)
        TOPIC_API_NAME_PROGRESS_1 -> SubscriptionTopic.Progress_1()
        TOPIC_API_NAME_PROGRESS_2 -> SubscriptionTopic.Progress_2()
        TOPIC_API_NAME_PROGRESS_3 -> SubscriptionTopic.Progress_3()
        TOPIC_API_NAME_PROGRESS_KINDERGARDEN -> SubscriptionTopic.Progress_kindergarden()
        TOPIC_API_NAME_PROGRESS_SCHOOL -> SubscriptionTopic.Progress_school()
        TOPIC_API_NAME_PROGRESS_LANDSCAPING -> SubscriptionTopic.Progress_landscaping()
        TOPIC_API_NAME_NEWS -> SubscriptionTopic.News()
        TOPIC_API_NAME_BLOG -> SubscriptionTopic.Blog()
        TOPIC_API_NAME_CONSCTRUCTION -> SubscriptionTopic.Construction()

    }
}










